package main

import (
	"cs"
	"q5"
	"strings"
	"sync"
	"time"
)

type FriendsMgr struct {
	cs.MsgHandlerImpl
	mu           sync.RWMutex
	users        map[string]*User         // accountId -> 用户
	searchCaches map[string]SearchCache   // SearchKeyword -> 好友搜索结果 []*User
	friendships  map[string][]*Friendship // accountId -> 好友关系列表 []*Friendship
	userCount    int                      // 用户总数
}

type SearchCache struct {
	Users        []*User
	LastModified time.Time
}

func (fm *FriendsMgr) init() {
	fm.userCount = 0
	fm.loadFromDB()
	fm.searchCaches = make(map[string]SearchCache)
}

func (fm *FriendsMgr) unInit() {
	// 1. Loop struct data
	// 2. Struct Data Persist to DB
}

func (fm *FriendsMgr) loadFromDB() {
	// 加载所有用户
	fm.loadUsers()
	// 加载好友关系表 列表
	fm.loadFriendships()
	// 加载等待验证好友请求 列表
	fm.loadPendingRequests()
	// 加载黑名单列表
	fm.loadBlacklist()
}

// 搜索指定用户
func (fm *FriendsMgr) searchByAccountId(accountId string) *User {
	if user, ok := fm.users[accountId]; ok {
		return user
	}
	return nil
}

// 搜索用户
func (fm *FriendsMgr) searchUsers(keyword string) []*User {
	if len(keyword) > SearchWord {
		return nil
	}
	if cachedResult, ok := fm.searchCaches[keyword]; ok {
		if time.Since(cachedResult.LastModified) <= 10*time.Hour {
			return cachedResult.Users
		}
	}

	listFriend := make([]*User, 0, MaxSearchResults)
	lowercaseQuery := strings.ToLower(keyword)
	for _, u := range fm.users {
		if strings.Contains(strings.ToLower(u.username), lowercaseQuery) {
			uEntity := &User{
				accountId: u.accountId,
				username:  u.username,
			}
			listFriend = append(listFriend, uEntity)
		}
		// 显示搜索结果 20条
		if len(listFriend) >= MaxSearchResults {
			break
		}
	}

	// Add to caches
	fm.searchCaches[keyword] = SearchCache{
		Users:        listFriend,
		LastModified: time.Now(),
	}
	fm.clearExpiredCaches()

	return listFriend
}

// addFriendRequest 添加好友请求 By default, account1Id is me
func (fm *FriendsMgr) addFriendRequest(account1Id string, account2Id string, cb func(errCode int32, errMsg string)) {
	user1 := fm.getUser(account1Id)
	user2 := fm.getUser(account2Id)
	if user1 == nil || user2 == nil {
		cb(1, "xxx")
	}

	found1 := user1.IsInBlacklist(account2Id)
	found2 := user2.IsInBlacklist(account1Id)
	if found1 == true || found2 == true {
		cb(1, "xxx")
	}

	// 已发送请求
	if _, ok := user1.friendRequest[account2Id]; ok {
		cb(1, "xxx")
	}

	// 自己好友已满
	if fm.getFriendCount(account1Id) >= MaxFriendMembers {
		cb(1, "xxx")
	}
	// 对方好友已满
	if fm.getFriendCount(account2Id) >= MaxFriendMembers {
		cb(1, "xxx")
	}
	// 对方待请求已满
	if fm.getFriendRequestCount(account2Id) >= MaxPendingFriendReqs {
		cb(1, "xxx")
	}

	fm.upsertFriendRequest(account1Id, account2Id, q5.ToString(FriendReqsStatusDefault), func(err error) {
		user1.AddFriendRequest(account2Id)
		user2.AddFriendRequest(account1Id)
	})
}

// acceptFriendRequest 接受好友请求
func (fm *FriendsMgr) acceptFriendRequest(account1Id string, account2Id string, cb func(errCode int32, errMsg string)) {
	user1 := fm.getUser(account1Id)
	user2 := fm.getUser(account2Id)
	if user1 == nil || user2 == nil {
		cb(1, "xxx")
	}

	if !user1.IsInReq(account2Id) {
		cb(1, "xxx")
	}

	if fm.getFriendCount(account1Id) >= MaxFriendMembers || fm.getFriendCount(account2Id) >= MaxFriendMembers {
		cb(1, "xxx")
	}

	// step1. update reqs
	fm.upsertFriendRequest(account2Id, account1Id, q5.ToString(FriendReqsStatusOk), func(err error) {
		fm.upsertFriendRequest(account1Id, account2Id, q5.ToString(FriendReqsStatusOk), func(err error) {
			// step2. insert friendship
			a1, a2 := swapAccountIds(account1Id, account2Id)
			fm.upsertFriendShip(a1, a2, FriendshipStatusOk, func(err error) {
				// step3. create a new friendship
				friendship := &Friendship{
					user1: fm.users[account1Id],
					user2: fm.users[account2Id],
				}
				fm.friendships[account1Id] = append(fm.friendships[account1Id], friendship)
				fm.friendships[account2Id] = append(fm.friendships[account2Id], friendship)
				// step4. remove requests
				user1.RemoveFriendRequest(account2Id)
				user2.RemoveFriendRequest(account1Id)
			})
		})
	})

}

// rejectFriendRequest 拒绝好友请求
func (fm *FriendsMgr) rejectFriendRequest(account1Id string, account2Id string, cb func(errCode int32, errMsg string)) {
	user1 := fm.getUser(account1Id)
	user2 := fm.getUser(account2Id)
	if user1 == nil || user2 == nil {
		cb(1, "xxx")
	}

	fm.upsertFriendRequest(account2Id, account1Id, q5.ToString(FriendReqsStatusReject), func(err error) {
		user1.RemoveFriendRequest(account2Id)
		user2.RemoveFriendRequest(account1Id)
	})
}

// deleteFriendShip 删除好友
func (fm *FriendsMgr) deleteFriendShip(account1Id, account2Id string, cb func(errCode int32, errMsg string)) {
	user1 := fm.getUser(account1Id)
	user2 := fm.getUser(account2Id)
	if user1 == nil || user2 == nil {
		cb(1, "xxx")
	}

	user1Friendships := fm.friendships[account1Id]
	user2Friendships := fm.friendships[account2Id]

	var found bool
	found, friendShipIndex := fm.findFriendshipIndex(account1Id, account2Id)
	if found {
		// 删除好友请求, upsert 不存在则新增，存在则替换值
		fm.upsertFriendRequest(account1Id, account2Id, q5.ToString(FriendReqsStatusDeleted), func(err error) {
			if err != nil {
				cb(1, "XXX")
				return
			}
			fm.friendships[account1Id] = append(user1Friendships[:friendShipIndex], user1Friendships[friendShipIndex+1:]...)
			found2, friendShipIndex2 := fm.findFriendshipIndex(account2Id, account1Id)
			if found2 {
				fm.upsertFriendRequest(account1Id, account2Id, q5.ToString(FriendReqsStatusDeleted), func(err error) {
					if err != nil {
						cb(1, "XXX")
						return
					}

					fm.friendships[account2Id] = append(user2Friendships[:friendShipIndex2], user2Friendships[friendShipIndex2+1:]...)
					// Delete friendship DB
					a1, a2 := swapAccountIds(account1Id, account2Id)
					fields := [][]string{{"is_delete_friendship", q5.ToString(FriendshipStatusDeleted)}}
					fm.updateFriendShip(a1, a2, fields, func(err error) {
						if err != nil {
							cb(1, "XXX")
							return
						}

						cb(0, "")
					})
				})
			}
		})
	}
}

// getFriendCount 好友数量
func (fm *FriendsMgr) getFriendCount(accountId string) int {
	if friends, ok := fm.friendships[accountId]; ok {
		return len(friends)
	}
	return 0
}

// getFriendRequestCount 等待请求数量
func (fm *FriendsMgr) getFriendRequestCount(accountId string) int {
	user := fm.getUser(accountId)
	if user != nil {
		return len(user.friendRequest)
	}
	return 0
}

// listFriend 我的好友列表
func (fm *FriendsMgr) listFriends(accountId string) []*User {
	// By default, Users member data count:10
	var users []*User
	for _, friendship := range fm.friendships[accountId] {
		if friendship.user1.accountId != accountId {
			uEntity := &User{
				accountId: friendship.user1.accountId,
				username:  friendship.user1.username,
			}
			users = append(users, uEntity)
		} else {
			uEntity := &User{
				accountId: friendship.user2.accountId,
				username:  friendship.user2.username,
			}
			users = append(users, uEntity)
		}
	}
	return users
}

// addBlacklist 加入黑名单
func (fm *FriendsMgr) addBlacklist(account1Id string, account2Id string, cb func(errCode int32, errMsg string)) {
	user1 := fm.getUser(account1Id)
	user2 := fm.getUser(account2Id)
	if user1 == nil || user2 == nil {
		cb(1, "xxx")
	}

	user1FriendBlackList := user1.GetBlacklist(account2Id)
	if user1FriendBlackList != nil && user1FriendBlackList.isRemoved != 0 {
		cb(1, "xxx")
	}

	if user1.GetBlacklistCount() >= MaxBlockedMembers {
		cb(1, "xxx")
	}

	fm.upsertBlacklist(account1Id, account2Id, BlacklistStatusDefault, func(err error) {
		if err != nil {
			user1.AddBlacklist(&FriendBlackList{
				accountId: account2Id,
				isRemoved: 0,
			})
			cb(0, "")
			return
		}
		cb(1, err.Error())
	})
}

// removeBlacklist 移除黑名单
func (fm *FriendsMgr) removeBlacklist(account1Id string, account2Id string, cb func(errCode int32, errMsg string)) {
	user1 := fm.getUser(account1Id)
	user2 := fm.getUser(account2Id)
	if user1 == nil || user2 == nil {
		cb(1, "xxx")
	}
	fm.upsertBlacklist(account1Id, account2Id, BlacklistStatusIsRemoved, func(err error) {
		if err != nil {
			user1.RemoveBlacklist(account2Id)
			cb(0, "")
			return
		}
		cb(1, err.Error())
	})
}

func (fm *FriendsMgr) addFriendshipToMap(accountID string, friendship *Friendship) {
	if fm.friendships[accountID] == nil {
		fm.friendships[accountID] = []*Friendship{friendship}
	} else {
		fm.friendships[accountID] = append(fm.friendships[accountID], friendship)
	}
}

func (fm *FriendsMgr) clearExpiredCaches() {
	expirationTime := time.Now().Add(-24 * time.Hour)
	for key, entry := range fm.searchCaches {
		if entry.LastModified.Before(expirationTime) {
			delete(fm.searchCaches, key)
		}
	}
}

func (fm *FriendsMgr) IsFriendShip(account1Id, Account2Id string) bool {
	var found bool
	for _, friendship := range fm.friendships[account1Id] {
		if friendship.user1.accountId == Account2Id || friendship.user2.accountId == Account2Id {
			return true
		}
	}
	return found
}

func (fm *FriendsMgr) GetFriendByAccountId(account1Id, Account2Id string) *User {
	for _, friendship := range fm.friendships[account1Id] {
		if friendship.user1.accountId == Account2Id {
			return friendship.user1
		}
		if friendship.user2.accountId == Account2Id {
			return friendship.user2
		}
	}
	return nil
}

func (fm *FriendsMgr) findFriendShipIndex(Account1Id, Account2Id string) int {
	// 通常 account1Id,指自己， account2Id 指对方
	if _, exists := fm.friendships[Account1Id]; exists {
		for i, friendship := range fm.friendships[Account1Id] {
			if friendship.user2.accountId == Account2Id {
				return i
			}
		}
	}
	return -1
}

func swapAccountIds(account1Id, account2Id string) (string, string) {
	if strings.Compare(account1Id, account2Id) > 0 {
		account1Id, account2Id = account2Id, account1Id
	}
	return account1Id, account2Id
}

func (fm *FriendsMgr) getUser(accountId string) *User {
	if user, ok := fm.users[accountId]; ok {
		return user
	}
	return nil
}

func (fm *FriendsMgr) getFriendships(accountId string) []*Friendship {
	if ships, ok := fm.friendships[accountId]; ok {
		return ships
	}
	return nil
}

func (fm *FriendsMgr) findFriendshipIndex(account1Id, account2Id string) (bool, int) {
	var found = false
	user1Friendships := fm.friendships[account1Id]
	for i, friendship := range user1Friendships {
		if friendship.user1.accountId == account2Id || friendship.user2.accountId == account2Id {
			found = true
			return found, i
		}
	}
	return found, -1
}
